#include "rclcpp/rclcpp.hpp"
#include "rclcpp_action/rclcpp_action.hpp"

#include "irobot_create_msgs/msg/interface_buttons.hpp"

#include "controls.h"

ButtonPressNode::ButtonPressNode()
: Node("button_press_node")
{
    interface_buttons_subscriber_ = this->create_subscription<irobot_create_msgs::msg::InterfaceButtons>(
        "/interface_buttons",
        rclcpp::SensorDataQoS(),
        std::bind(&ButtonPressNode::result_callback, this, std::placeholders::_1)
    );
}

void ButtonPressNode::bt1_wait()
{
    RCLCPP_INFO(this->get_logger(), "Wait for button 1...");
    button1 = false;
    while (!button1) {}
    button1 = false;
}

void ButtonPressNode::bt2_wait()
{
    RCLCPP_INFO(this->get_logger(), "Wait for button 2...");
    button2 = false;
    while (!button2) {}
    button2 = false;
}

void ButtonPressNode::result_callback(const irobot_create_msgs::msg::InterfaceButtons::SharedPtr result)
{
    if (result->button_1.is_pressed) {
        button1 = true;
    }

    if (result->button_2.is_pressed) {
        button2 = true;
    }

    if (result->button_power.is_pressed) {

    }
}

void ButtonPressNode::kill()
{
    RCLCPP_INFO(this->get_logger(), "ButtonPressNode killed");
    rclcpp::shutdown();
}